package cd.casic.ci.log.scm.service.impl;


import cd.casic.ci.log.scm.dal.pojo.oauth.GitToken;
import cd.casic.ci.log.scm.service.scm.IGitService;
import cd.casic.framework.commons.util.encrypt.AESUtil;
import cd.casic.ci.log.scm.dal.dao.RepositoryGitTokenMapper;
import cd.casic.ci.log.scm.dal.dto.RepositoryGitToken;
import cd.casic.ci.log.scm.service.IGitOauthService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.redisson.RedissonRedLock;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Service;

import java.time.ZoneId;

@Service
@Slf4j
public class GitOauthServiceImpl implements IGitOauthService {

    private static final String aesKey = "";

    @Resource
    private IGitService gitService;

    @Resource
    private RepositoryGitTokenMapper repositoryGitTokenMapper;

    @Resource
    private RedissonClient redissonClient;

    @Override
    public GitToken getAccessToken(String userId) {
        GitToken accessToken = doGetAccessToken(userId);
        if (accessToken == null) {
            return null;
        }

        if (isTokenExpire(accessToken)) {
            log.info("try to refresh the git token of user({})", userId);
            RLock rLock = redissonClient.getLock("OAUTH_REFRESH_TOKEN_" + userId);
            RedissonRedLock redisLock = new RedissonRedLock(rLock);
            try {
                redisLock.lock();
                GitToken newAccessToken = doGetAccessToken(userId);
                if (newAccessToken.getExpiresIn() * 1000 <= System.currentTimeMillis() - 1800 * 1000) {
                    return refreshToken(userId, newAccessToken);
                } else {
                    return newAccessToken;
                }
            } finally {
                redisLock.unlock();
            }
        } else {
            return accessToken;
        }
    }

    private GitToken refreshToken(String userId, GitToken gitToken) {
        GitToken token = gitService.refreshToken(userId, gitToken);
        saveAccessToken(userId, token);
        token.setAccessToken(AESUtil.decrypt(aesKey, token.getAccessToken()));
        token.setAccessToken(AESUtil.decrypt(aesKey, token.getRefreshToken()));
        return token;
    }

    @Override
    public int saveAccessToken(String userId, GitToken tGitToken) {
        tGitToken.setAccessToken(AESUtil.encrypt(aesKey, tGitToken.getAccessToken()));
        tGitToken.setRefreshToken(AESUtil.encrypt(aesKey, tGitToken.getRefreshToken()));
        RepositoryGitToken repositoryGitToken = new RepositoryGitToken();
        repositoryGitToken.setUserId(userId);
        repositoryGitToken.setAccessToken(tGitToken.getAccessToken());
        repositoryGitToken.setRefreshToken(tGitToken.getRefreshToken());
        repositoryGitToken.setTokenType(tGitToken.getTokenType());
        repositoryGitToken.setExpiresIn(tGitToken.getExpiresIn());
        return repositoryGitTokenMapper.insert(repositoryGitToken);
    }

    private Boolean isTokenExpire(GitToken accessToken) {
        // 提前半个小时刷新token
        return (accessToken.getCreateTime() != null ? accessToken.getCreateTime() : 0) + accessToken.getExpiresIn() * 1000 - 1800 * 1000 <= System.currentTimeMillis();
    }

    private GitToken doGetAccessToken(String userId) {
        RepositoryGitToken token = repositoryGitTokenMapper.selectOne(RepositoryGitToken::getUserId, userId);
        if (token == null) {
            return null;
        } else {
            return new GitToken(
                    AESUtil.decrypt(aesKey, token.getAccessToken()),
                    AESUtil.decrypt(aesKey, token.getRefreshToken()),
                    token.getTokenType(), token.getExpiresIn(),
                    token.getCreateTime().atZone(ZoneId.systemDefault()).toInstant().toEpochMilli()
            );
        }
    }
}
